(* Reading and writing a simple Blog *)

let blog_dir = "/tmp/blog"
let last_entries = 5			(* display that many last entries *)
;;
(* Each blog is a file whose name is of the form nnnnnn.txt
   where nnnnnn is 0-padded number *)

(* Read the file into the buffer *)
let read_file buf fname =
  let cin = open_in_bin fname in
  let len = in_channel_length cin in
  let str = String.create len in
  really_input cin str 0 len;
  close_in cin;
  Buffer.add_string buf str;
  Buffer.add_string buf "\n\n"
;;

let rec take n = function
  | [] -> []
  | _ when n = 0 -> []
  | (h::t) -> h::(take (pred n) t)
;;
let read_blog () =
  let buf = Buffer.create 1024 in
  let blogs = Sys.readdir blog_dir in
  let () = Array.sort (fun x y -> - (compare x y)) blogs in
  let blogs = Array.sub blogs 0 (min last_entries (Array.length blogs)) in
  Array.iter (fun fname -> read_file buf (Filename.concat blog_dir fname)) 
    blogs;
  Buffer.contents buf
;;

let write_blog str =
  let rec maxno = function
    | [] -> 0
    | (h::t) -> let hn = Scanf.sscanf h "%d.txt" (fun x -> x) in
	        let tn = maxno t in
		max hn tn
  in
  let rec new_fname n tries =
    let fname = Filename.concat blog_dir (Printf.sprintf "%06d.txt" n) in
    try open_out_gen [Open_wronly; Open_creat; Open_excl; Open_binary] 438
	fname
    with Sys_error e -> if tries > 0 then new_fname (succ n) (pred tries)
	                else raise (Sys_error e)
  in
  let cout = 
    new_fname (succ (maxno (Array.to_list (Sys.readdir blog_dir)))) 5 in
  output_string cout str;
  close_out cout
;;
let markup username title body =
  Printf.sprintf "User %s\n%s\n%s\n" username title body
;;
