(* Simple CGI interface *)

(* CGI environment *)
type hvar = string
module HEnv : Map.S with type key = hvar
type henv = string HEnv.t

val hlocate : hvar -> henv -> string option

(* Data type of values being exchanged with a Web client *)


(* Input validation routines *)
type ('a,'b) either = Left of 'a | Right of 'b
type 'a validation_fn = string option -> (string,'a) either
val vstring : string validation_fn
val vint    : int    validation_fn
val vfloat  : float  validation_fn
val vbool   : bool   validation_fn

(* Given henv, the name of the expected variable and the validation
   function, extract the desired input and validate it.
   Invoke inquirer to report an error and ask for a corrected reply.
*)
val validate : (henv -> henv) -> hvar -> henv -> 'a validation_fn -> 'a


(* Obtain the CGI environment *)
val get_form_env : unit -> henv

(* A base error handler, to be used when none of the HTTP headers are sent. 
   It should send all the headers and 'reset' (because this is a CGI script, 
   reset means finish)
*)
val base_error_handler : string -> 'a

(* Decode QUERY_STRING or www-form-urlencoded data *)
val url_unquote_collate : string -> henv

(* Encode a string escaping all HTML-significant characters *)
val html_encode_string : string -> string

(* 
  send_form cout henv template_file_name
  Send the template to stdout substituting the variables according to henv
*)
val send_form : out_channel -> henv -> string -> unit 

(* Various utilities *)
val maybe : (unit -> 'w) -> ('a -> 'w) -> 'a option -> 'w

val splitby : string -> char -> string list
