(* Continuation-based CGI interface *)
open Cgi

module type KCGI = sig
  (* This is like writing followed by reading. We combine them
     for sake of input validation.
  *)
  (* This is sort of like get_text: write the string and return the reply,
     represented by henv
  *)
  val inquire : 
    string -> 				(* template *)
    (hvar * string) list ->		(* outputs*)
    henv				(* information about inputs *)

  (* a particular case of the above when we report the final
     result and no further input is requested. We are done.
  *)
  val inquire_finish : 
    string -> 				(* template *)
    (hvar * string) list ->		(* outputs*)
    'a				        (* does not return *)

  (* Extract a particular answer from henv. Note henv contains a particular
     key that identifies the template used to solicit the answer. If the
     answer turns out invalid, we use the template to report the problem 
     and ask for a corrected reply.
  *)
  val answer : 
    hvar ->				(* the name of the desired form var*)
    henv -> 				(* representation of the answer *)
    'a validation_fn ->			(* validation function  *)
    'a

  val in_transaction : 
      (hvar * string) list -> ((hvar * string) list -> 'a) -> 'a

  (* An exception to raise when trying to continue an already closed
     transaction *)
  exception TX_Gone
end

(* A special HVAR to identify the template used to solicit the input *)
let hvar_template_name = "&TEMPLATE&"


module type MAIN = functor (S:KCGI) -> sig
  val template_dir : string
  val main : unit -> 'a
end

module type RUN = functor (S:MAIN) -> sig
  val run : unit -> unit		(* Normally, does not return *)
end
