(* Continuation-based CGI interface *)
(* This file stores the continuations in memory. This is suitable for
   a constantly-running server such as FastCGI
*)

open Cgi
open Delimcc
open Kcgi

let gensym =
  let counter = ref 0 in
  fun () -> 
    let r = "g" ^ (string_of_int !counter) in
    incr counter; r

type cgi_result = 
    Res of (string * (henv -> cgi_result)) option * string *
    (henv,(hvar * string) list) either


let p0 : cgi_result prompt = new_prompt ()

module KCGI_M = struct 

(* This is like writing followed by reading. We combine them
   for the sake of input validation.
*)

let do_inquire template env =
  let send k = 
      let klabel = gensym () in
      Res (Some (klabel,k),template,env) in
  let henv = shift p0 send in
  HEnv.add hvar_template_name template henv 

let inquire template outputs =
  do_inquire template (Right outputs)

let inquire_finish template outputs =
    abort p0 (Res (None, template, Right outputs))

let answer hvar henv validfn = 
  validate (fun henv -> 
    do_inquire (HEnv.find hvar_template_name henv) (Left henv)) 
    hvar henv validfn
end


module Run(M:MAIN) = struct

 module Main = M(KCGI_M)
 open Main

 let run () = 
  let rec loop jobqueue =
    let (k,template,henv) =
      try
	(* let henv = get_form_env () in *)
	let henv = url_unquote_collate (read_line ()) in
	let Res (k,template,env) =
	  match 
	    try Some (List.assoc (HEnv.find "klabel" henv) jobqueue)
	    with Not_found -> None  with
	  | None -> push_prompt p0 (fun () -> main (); failwith "na")
	  | Some k -> k henv in
	let henv = match env with
	| Left henv' -> HEnv.fold HEnv.add henv' henv 
	| Right env  ->
	    List.fold_left (fun m (k,v) -> HEnv.add k v m) henv env in
	let henv = match k with
	| Some (klabel,_) -> HEnv.add "klabel" klabel henv
	| None -> henv in
	(k,template,henv)
      with e -> base_error_handler (Printexc.to_string e) in
    (* Now we are creating output *)
    print_endline "Content-type: text/html\n";
    send_form stdout henv (Filename.concat template_dir template);
    flush_all ();
    match k with
    | Some job -> loop (job::jobqueue)
    | None -> loop jobqueue
 in loop []
end

