(* Continuation-based CGI interface *)
(* This file uses the siple terminal IO. No continuations,
   except those captured by the OS when we do read.
*)

open Cgi
open Kcgi

 (* This is like writing followed by reading. We combine them
   for the sake of input validation. This function also should remind
   one of gettext...
 *)

module KCGI_T = struct

 let template_dir = ref ""

 let do_inquire template henv =
   send_form stdout henv (Filename.concat !template_dir template);
   flush_all ();
   let henv = url_unquote_collate (read_line ()) in
   HEnv.add hvar_template_name template henv 

  let inquire template outputs =
    let henv = 
      List.fold_left (fun m (k,v) -> HEnv.add k v m) HEnv.empty outputs in
    do_inquire template henv


  let inquire_finish template outputs = 
    let _ = inquire template outputs in exit 0

  let answer hvar henv validfn = 
    validate (fun henv -> 
      do_inquire (HEnv.find hvar_template_name henv) henv)
      hvar henv validfn

  let in_transaction env fn = failwith "in_transaction is not implemented here"

  (* An exception to raise when trying to continue an already closed
     transaction *)
  exception TX_Gone
end

module Run(M:MAIN) = struct

 module Main = M(KCGI_T)
 open Main

 let () = KCGI_T.template_dir := Main.template_dir

 let run () = 
   try
     main ()
   with e -> base_error_handler (Printexc.to_string e)
end

