(* CGI from Christian Queinnec's ICFP00 paper. Fig 1.
   We only replaced no-longer used francs with yens.

   This code is for comparison only: it is page-centric rather
   then computation-centric. This is the usual way of writing CGI
   scripts, using explicit continuations. One can see clearly the 
   inversion of control, the lack of lexical scope, the need for repeated
   validation. The inconvenience becomes even more pronounced
   if instead of the dispatch on the value of klabel, we split this script
   into three (as is common).
*)
open Cgi

(*
#load "cgi.cmo";;
*)


module MAIN = struct

  (* Where all the template files are *)
  let template_dir = "/home/oleg/Cache/ometa-cvs/XXCC-natdyn/delimcc";;
  (*
  let template_dir = "/home/oleg/Cache/ometa-cvs/XXCC-natdyn/delimcc/tty";;
  *)

 let send template env =
   let henv = List.fold_left (fun m (k,v) -> HEnv.add k v m) HEnv.empty env in
   print_endline "Content-type: text/html\n";
   send_form stdout henv (Filename.concat template_dir template);
   flush_all ()

 let validate v vf = 
   match vf v with
   | Right v -> v
   | Left msg -> failwith msg		(* Need error handling here *)

 let main () =
  let henv = get_form_env () in
  match hlocate "klabel" henv with
  | None -> send "currency_read_rate.html" []
  | Some "got-rate" -> 
      (match (hlocate "curr-name" henv, hlocate "rate" henv) with
      |	(Some curr_name, (Some rate as rv)) ->
	  let _ = validate rv vfloat in
	  send "currency_read_yen.html" 
	    [("curr-name",curr_name); ("rate",rate)]
      |	_ -> failwith "need error handling")
  | Some "got-amount" ->
      (match (hlocate "curr-name" henv, hlocate "rate" henv,
	      hlocate "curr-amount" henv) with
      |	(Some curr_name, (Some _ as rv), (Some _ as amv)) ->
	  let curr_rate = validate rv vfloat in (* validate again *)
	  let amount = validate amv vfloat in
	  let yen_amount = amount /. curr_rate in
	  send "currency_result.html"
	    [("curr-name",curr_name);("rate",string_of_float curr_rate);
	     ("curr-amount",string_of_float amount); 
	     ("yen-amount", string_of_float yen_amount)]
      |	_ -> failwith "need error handling")
  | _ -> failwith "need error handling";
  exit 0
end


